@@ -93,6 +93,11 @@ std::ostream& multi::operator<<(std::ostream& out, multi::Code code)
     case multi::Code::bad_socket: out << "curl::multi::Code::bad_socket"; break;
     case multi::Code::unknown_option: out << "curl::multi::Code::unknown_option"; break;
     case multi::Code::added_already: out << "curl::multi::Code::added_already"; break;
+    case multi::Code::recursive_api_call: out << "curl::multi::Code::recursive_api_call"; break;
+    case multi::Code::wakeup_failure: out << "curl::multi::Code::wakeup_failure"; break;
+    case multi::Code::bad_function_argument: out << "curl::multi::Code::bad_function_argument"; break;
+    case multi::Code::aborted_by_callback: out << "curl::multi::Code::aborted_by_callback"; break;
+    default: out << "curl::multi::Code::unknown"; break;
     }
 
     return out;
@@ -387,32 +392,22 @@ void multi::Handle::Private::Timeout::Private::cancel()
 
 void multi::Handle::Private::Timeout::Private::async_wait_for(const std::weak_ptr<Handle::Private>& context, const std::chrono::milliseconds& ms)
 {
-    if (ms.count() > 0)
+    std::weak_ptr<Private> self{shared_from_this()};
+    timer.expires_from_now(boost::posix_time::milliseconds{ms.count()});
+    timer.async_wait([self, context](const boost::system::error_code& ec)
     {
-        std::weak_ptr<Private> self{shared_from_this()};
-        timer.expires_from_now(boost::posix_time::milliseconds{ms.count()});
-        timer.async_wait([self, context](const boost::system::error_code& ec)
-        {
-            if (ec)
-                return;
-
-            if (auto spc = context.lock())
-            {
-                if (auto sp = self.lock())
-                {
-                    std::lock_guard<std::mutex> lg(spc->guard);
-                    sp->handle_timeout(spc);
-                }
-            }
-        });
-    } else if (ms.count() == 0)
-    {
-        auto sp = context.lock();
-        if (not sp)
+        if (ec)
             return;
 
-        handle_timeout(sp);
-    }
+        if (auto spc = context.lock())
+        {
+            if (auto sp = self.lock())
+            {
+                std::lock_guard<std::mutex> lg(spc->guard);
+                sp->handle_timeout(spc);
+            }
+        }
+    });
 }
 
 void multi::Handle::Private::Timeout::Private::handle_timeout(const std::shared_ptr<Handle::Private>& context)
@@ -428,8 +423,9 @@ int multi::Handle::Private::timer_callback(
         long timeout_ms,
         void* cookie)
 {
-    if (timeout_ms < 0)
-        return -1;
+    /* -1 means we should just delete our timer. */
+    if (timeout_ms == -1)
+        return 0;
 
     auto holder = static_cast<Private::Holder*>(cookie);
 
